@@ -57,8 +57,7 @@ class PlotPanel(QtWidgets.QWidget):
         self.widget_range_y.range_changed.connect(self.on_range_changed)
 
     def __getstate__(self):
-        feats_srt = self.pipeline.get_features(
-            scalar=True, label_sort=True, plot_id=self.current_plot.identifier)
+        feats_srt = self.get_features()
 
         rx = self.widget_range_x.__getstate__()
         ry = self.widget_range_y.__getstate__()
@@ -130,8 +129,7 @@ class PlotPanel(QtWidgets.QWidget):
     def __setstate__(self, state):
         if self.current_plot.identifier != state["identifier"]:
             raise ValueError("Plot identifier mismatch!")
-        feats_srt = self.pipeline.get_features(
-            scalar=True, label_sort=True, plot_id=self.current_plot.identifier)
+        feats_srt = self.get_features()
         toblock = [
             self.comboBox_axis_x,
             self.comboBox_axis_y,
@@ -177,12 +175,15 @@ class PlotPanel(QtWidgets.QWidget):
         hue_index = self.comboBox_marker_hue.findData(sca["marker hue"])
         self.comboBox_marker_hue.setCurrentIndex(hue_index)
         self.doubleSpinBox_marker_size.setValue(sca["marker size"])
-        feat_index = feats_srt.index(sca["hue feature"])
+        if sca["hue feature"] in feats_srt:
+            feat_index = feats_srt.index(sca["hue feature"])
+        else:
+            feat_index = 0  # feature not available in datasets
         self.comboBox_marker_feature.setCurrentIndex(feat_index)
         color_index = COLORMAPS.index(sca["colormap"])
         self.comboBox_colormap.setCurrentIndex(color_index)
         self.checkBox_event_count.setChecked(sca["show event count"])
-        self.spinBox_alpha.setValue(sca["marker alpha"]*100)
+        self.spinBox_alpha.setValue(int(sca["marker alpha"]*100))
         if sca["marker hue"] == "feature":
             self._set_range_feat_state(sca["hue feature"], sca["hue min"],
                                        sca["hue max"])
@@ -397,6 +398,18 @@ class PlotPanel(QtWidgets.QWidget):
             ids = []
         return ids
 
+    def get_features(self):
+        """Wrapper around pipeline with default features if empty"""
+        feats_srt = self.pipeline.get_features(
+            scalar=True, label_sort=True, plot_id=self.current_plot.identifier)
+        if len(feats_srt) == 0:
+            # fallback (nothing in the pipeline)
+            features = dclab.dfn.scalar_feature_names
+            labs = [dclab.dfn.get_feature_label(f) for f in features]
+            lf = sorted(zip(labs, features))
+            feats_srt = [it[1] for it in lf]
+        return feats_srt
+
     def on_axis_changed(self):
         gen = self.__getstate__()["general"]
         if self.sender() == self.comboBox_axis_x:
@@ -510,20 +523,21 @@ class PlotPanel(QtWidgets.QWidget):
             for cb in [self.comboBox_axis_x,
                        self.comboBox_axis_y,
                        self.comboBox_marker_feature]:
+                # get the features currently available
+                feats_srt = self.get_features()
                 cb.blockSignals(True)
+                # remember previous selection if possible
                 if cb.count:
                     # remember current selection
                     curfeat = cb.currentData()
+                    if curfeat not in feats_srt:
+                        curfeat = None
                 else:
                     curfeat = None
                 # repopulate
                 cb.clear()
-                feats_srt = self.pipeline.get_features(
-                    scalar=True,
-                    label_sort=True,
-                    plot_id=self.current_plot.identifier)
                 for feat in feats_srt:
-                    cb.addItem(dclab.dfn.feature_name2label[feat], feat)
+                    cb.addItem(dclab.dfn.get_feature_label(feat), feat)
                 if curfeat is not None:
                     # write back current selection
                     curidx = feats_srt.index(curfeat)
