@@ -1,3 +1,4 @@
+import os
 import pathlib
 import pkg_resources
 import signal
@@ -5,31 +6,30 @@ import sys
 import traceback
 import webbrowser
 
-import appdirs
 import dclab
 import h5py
 import numpy
 import scipy
 
-from PyQt5 import uic, QtCore, QtWidgets
+from PyQt5 import uic, QtCore, QtGui, QtWidgets
 import pyqtgraph as pg
 
 from . import analysis
+from . import bulk
 from . import compute
 from . import dcor
 from . import export
 from . import pipeline_plot
+from . import preferences
 from . import quick_view
 from . import update
 from . import widgets
 
 from .. import pipeline
 from .. import session
-from .. import settings
 
 
 from .._version import version as __version__
-from shapeout2.gui.widgets.wait_cursor import ShowWaitCursor, show_wait_cursor
 
 
 # global plotting configuration parameters
@@ -39,6 +39,13 @@ pg.setConfigOption("antialias", True)
 pg.setConfigOption("imageAxisOrder", "row-major")
 
 
+# set Qt icon theme search path
+QtGui.QIcon.setThemeSearchPaths([
+    os.path.join(pkg_resources.resource_filename("shapeout2", "img"),
+                 "icon-theme")])
+QtGui.QIcon.setThemeName(".")
+
+
 class ShapeOut2(QtWidgets.QMainWindow):
     plots_changed = QtCore.pyqtSignal()
 
@@ -55,48 +62,51 @@ class ShapeOut2(QtWidgets.QMainWindow):
         # update check
         self._update_thread = None
         self._update_worker = None
+        # Settings are stored in the .ini file format. Even though
+        # `self.settings` may return integer/bool in the same session,
+        # in the next session, it will reliably return strings. Lists
+        # of strings (comma-separated) work nicely though.
+        QtCore.QCoreApplication.setOrganizationName("Zellmechanik-Dresden")
+        QtCore.QCoreApplication.setOrganizationDomain("zellmechanik.com")
+        QtCore.QCoreApplication.setApplicationName("shapeout2")
+        QtCore.QSettings.setDefaultFormat(QtCore.QSettings.IniFormat)
         #: Shape-Out settings
-        self.settings = settings.SettingsFile()
+        self.settings = QtCore.QSettings()
+        self.settings.setIniCodec("utf-8")
         # Register user-defined DCOR API Key in case the user wants to
         # open a session with private data.
-        api_key = self.settings.get_string("dcor api key")
+        api_key = self.settings.value("dcor/api key", "")
         dclab.rtdc_dataset.fmt_dcor.APIHandler.add_api_key(api_key)
         #: Analysis pipeline
         self.pipeline = pipeline.Pipeline()
         # GUI
         self.setWindowTitle("Shape-Out {}".format(__version__))
-        # Disable native menubar (e.g. on Mac)
+        # Disable native menu bar (e.g. on Mac)
         self.menubar.setNativeMenuBar(False)
         # File menu
+        self.actionLoadDataset.triggered.connect(self.add_dataslot)
+        self.actionLoadDCOR.triggered.connect(self.on_action_dcor)
         self.actionClearSession.triggered.connect(self.on_action_clear)
         self.actionOpenSession.triggered.connect(self.on_action_open)
         self.actionQuit.triggered.connect(self.on_action_quit)
         self.actionSaveSession.triggered.connect(self.on_action_save)
-        # Help menu
-        self.actionDocumentation.triggered.connect(self.on_action_docs)
-        self.actionSoftware.triggered.connect(self.on_action_software)
-        # developer mode
-        self.actionDeveloperMode.setChecked(
-            self.settings.get_bool("developer mode"))
-        self.actionDeveloperMode.triggered.connect(self.on_action_develop)
-        # check for updates
-        do_update = self.settings.get_bool("check update")
-        self.actionCheckUpdate.setChecked(do_update)
-        self.actionCheckUpdate.triggered.connect(self.on_action_check_update)
-        self.on_action_check_update(do_update)  # check for updates if True
-        self.actionAbout.triggered.connect(self.on_action_about)
+        # Edit menu
+        self.actionPreferences.triggered.connect(self.on_action_preferences)
+        # Compute menu
+        self.actionComputeStatistics.triggered.connect(
+            self.on_action_compute_statistics)
+        self.actionComputeEmodulus.triggered.connect(
+            self.on_action_compute_emodulus)
         # Export menu
         self.actionExportData.triggered.connect(self.on_action_export_data)
         self.actionExportFilter.triggered.connect(self.on_action_export_filter)
         self.actionExportPlot.triggered.connect(self.on_action_export_plot)
         # Import menu
         self.actionImportFilter.triggered.connect(self.on_action_import_filter)
-        # Compute menu
-        self.actionComputeStatistics.triggered.connect(
-            self.on_action_compute_statistics)
-        # Initially hide buttons
-        self.pushButton_preset_load.hide()
-        self.pushButton_preset_save.hide()
+        # Help menu
+        self.actionDocumentation.triggered.connect(self.on_action_docs)
+        self.actionSoftware.triggered.connect(self.on_action_software)
+        self.actionAbout.triggered.connect(self.on_action_about)
         # Subwindows
         self.subwindows = {}
         # Subwindows for plots
@@ -105,15 +115,11 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.init_quick_view()
         self.init_analysis_view()
         self.mdiArea.cascadeSubWindows()
-        self.showMaximized()
-        # DCOR Dialog
-        self.actionLoadDCOR.triggered.connect(self.on_action_dcor)
         # BLOCK MATRIX (wraps DataMatrix and PlotMatrix)
         # BlockMatrix appearance
         self.toolButton_dm.clicked.connect(self.on_data_matrix)
         self.splitter.splitterMoved.connect(self.on_splitter)
         # BlockMatrix Actions
-        self.actionLoadDataset.triggered.connect(self.add_dataslot)
         self.actionNewFilter.triggered.connect(self.add_filter)
         self.actionNewPlot.triggered.connect(self.add_plot)
         self.block_matrix.toolButton_load_dataset.clicked.connect(
@@ -152,17 +158,26 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.widget_ana_view.plot_changed.connect(self.adopt_plot)
         # slot signals
         self.widget_ana_view.slot_changed.connect(self.adopt_slot)
-        # check pyqtgraph version
-        self._check_pg_version()
         # if "--version" was specified, print the version and exit
         if "--version" in sys.argv:
             print(__version__)
             QtWidgets.QApplication.processEvents()
             sys.exit(0)
+        # check pyqtgraph version
+        self._check_pg_version()
+        # check for updates
+        do_update = int(self.settings.value("check for updates", 1))
+        self.on_action_check_update(do_update)
+        # finalize
+        self.show()
+        self.raise_()
+        self.activateWindow()
+        self.showMaximized()
+        self.setWindowState(QtCore.Qt.WindowState.WindowActive)
 
     def _check_pg_version(self):
         """Tells the user if the pyqtgraph version is not correct"""
-        if (self.settings.get_bool("check pgversion")
+        if (int(self.settings.value("advanced/check pyqtgraph version", 1))
                 and pg.__version__ != "0.11.0.dev0+g1c63ae5"):
             msg = QtWidgets.QMessageBox()
             msg.setIcon(QtWidgets.QMessageBox.Warning)
@@ -177,7 +192,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             )
             msg.exec_()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(dict)
     def adopt_filter(self, filt_state):
         filt_id = filt_state["identifier"]
@@ -197,7 +212,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             raise ValueError("Filter not in pipeline: {}".format(filt_id))
         self.adopt_pipeline(state)
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(dict)
     def adopt_pipeline(self, pipeline_state):
         # If the number of subplots within a plot changed, update the
@@ -289,7 +304,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.mdiArea.update()
         self.subwindows["analysis_view"].update()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(dict)
     def adopt_plot(self, plot_state):
         plot_id = plot_state["identifier"]
@@ -302,7 +317,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             raise ValueError("Plot not in pipeline: {}".format(plot_id))
         self.adopt_pipeline(state)
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(dict)
     def adopt_slot(self, slot_state):
         slot_id = slot_state["identifier"]
@@ -322,7 +337,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             raise ValueError("Slot not in pipeline: {}".format(slot_id))
         self.adopt_pipeline(state)
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot()
     def add_dataslot(self, paths=None, is_dcor=False):
         """Adds a dataslot to the pipeline
@@ -340,7 +355,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             fnames, _ = QtWidgets.QFileDialog.getOpenFileNames(
                 parent=self,
                 caption="Select an RT-DC measurement",
-                directory=self.settings.get_path(name="rtdc import dataset"),
+                directory=self.settings.value("paths/add dataset", ""),
                 filter="RT-DC Files (*.rtdc)")
         else:
             fnames = paths
@@ -356,8 +371,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
                 path = fn
             else:
                 path = pathlib.Path(fn)
-                self.settings.set_path(wd=path.parent,
-                                       name="rtdc import dataset")
+                self.settings.setValue("paths/add dataset", str(path.parent))
             # add a filter if we don't have one already
             if self.pipeline.num_filters == 0:
                 self.add_filter()
@@ -454,19 +468,23 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.mdiArea.addSubWindow(sub)
 
     def on_action_about(self):
+        gh = "ZELLMECHANIK-DRESDEN/ShapeOut2"
+        rtd = "shapeout2.readthedocs.io"
         about_text = "Shape-Out 2 is the successor of Shape-Out, " \
             + "a graphical user interface for the analysis and " \
-            + "visualization of RT-DC data sets.\n\n" \
-            + "Author: Paul Müller\n" \
-            + "Code: https://github.com/ZELLMECHANIK-DRESDEN/ShapeOut2\n" \
-            + "Documentation: https://shapeout2.readthedocs.io"
+            + "visualization of RT-DC data sets.<br><br>" \
+            + "Author: Paul Müller<br>" \
+            + "GitHub: " \
+            + "<a href='https://github.com/{gh}'>{gh}</a><br>".format(gh=gh) \
+            + "Documentation: " \
+            + "<a href='https://{rtd}'>{rtd}</a><br>".format(rtd=rtd)
         QtWidgets.QMessageBox.about(self,
                                     "Shape-Out {}".format(__version__),
                                     about_text)
 
     @QtCore.pyqtSlot(bool)
     def on_action_check_update(self, b):
-        self.settings.set_bool("check update", b)
+        self.settings.setValue("check for updates", int(b))
         if b and self._update_thread is None:
             self._update_thread = QtCore.QThread()
             self._update_worker = update.UpdateWorker()
@@ -513,14 +531,10 @@ class ShapeOut2(QtWidgets.QMainWindow):
         dlg = compute.ComputeStatistics(self, pipeline=self.pipeline)
         dlg.exec()
 
-    @QtCore.pyqtSlot(bool)
-    def on_action_develop(self, b):
-        self.settings.set_bool("developer mode", b)
-        msg = QtWidgets.QMessageBox()
-        msg.setIcon(QtWidgets.QMessageBox.Information)
-        msg.setText("Please restart Shape-Out for the changes to take effect.")
-        msg.setWindowTitle("Restart Shape-Out")
-        msg.exec_()
+    def on_action_compute_emodulus(self):
+        dlg = bulk.BulkActionEmodulus(self, pipeline=self.pipeline)
+        dlg.pipeline_changed.connect(self.adopt_pipeline)
+        dlg.exec()
 
     @QtCore.pyqtSlot()
     def on_action_clear(self, assume_yes=False):
@@ -580,7 +594,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             search_paths = []
             while True:
                 try:
-                    with ShowWaitCursor():
+                    with widgets.ShowWaitCursor():
                         session.open_session(path, self.pipeline, search_paths)
                 except session.DataFileNotFoundError as e:
                     missds = "\r".join([str(pp) for pp in e.missing_paths])
@@ -602,6 +616,11 @@ class ShapeOut2(QtWidgets.QMainWindow):
                     break
             self.reload_pipeline()
 
+    def on_action_preferences(self):
+        """Show the DCOR import dialog"""
+        dlg = preferences.Preferences(self)
+        dlg.exec()
+
     def on_action_quit(self):
         """Determine what happens when the user wants to quit"""
         if self.pipeline.slots or self.pipeline.filters:
@@ -618,8 +637,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             session.save_session(path, self.pipeline)
 
     def on_action_software(self):
-        libs = [appdirs,
-                dclab,
+        libs = [dclab,
                 h5py,
                 numpy,
                 pg,
@@ -631,13 +649,14 @@ class ShapeOut2(QtWidgets.QMainWindow):
         for lib in libs:
             sw_text += "- {} {}\n".format(lib.__name__, lib.__version__)
         sw_text += "- PyQt5 {}\n".format(QtCore.QT_VERSION_STR)
+        sw_text += "\n Breeze icon theme by the KDE Community (LGPL)."
         if hasattr(sys, 'frozen'):
             sw_text += "\nThis executable has been created using PyInstaller."
         QtWidgets.QMessageBox.information(self,
                                           "Software",
                                           sw_text)
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot()
     def on_data_matrix(self):
         """Show/hide data matrix (User clicked Data Matrix button)"""
@@ -667,7 +686,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             # adjusts QuickView size correctly
             self.widget_quick_view.on_tool()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(str)
     def on_modify_filter(self, filt_id):
         self.widget_ana_view.tabWidget.setCurrentIndex(2)
@@ -679,7 +698,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.mdiArea.update()
         self.subwindows["analysis_view"].update()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(str)
     def on_modify_plot(self, plot_id):
         self.widget_ana_view.tabWidget.setCurrentIndex(3)
@@ -691,7 +710,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.mdiArea.update()
         self.subwindows["analysis_view"].update()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(str)
     def on_modify_slot(self, slot_id):
         self.widget_ana_view.tabWidget.setCurrentIndex(1)
@@ -703,7 +722,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
         self.mdiArea.update()
         self.subwindows["analysis_view"].update()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(bool)
     def on_quickview(self, view=True):
         """Show/Hide QuickView (User clicked the QuickView button)"""
@@ -714,7 +733,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
         if view:
             self.subwindows["quick_view"].update()
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot()
     def on_quickview_refresh(self):
         """Refresh quickview with the currently shown dataset"""
@@ -723,7 +742,7 @@ class ShapeOut2(QtWidgets.QMainWindow):
             self.on_quickview_show_dataset(slot_index, filt_index,
                                            update_ana_filter=False)
 
-    @show_wait_cursor
+    @widgets.show_wait_cursor
     @QtCore.pyqtSlot(int, int)
     def on_quickview_show_dataset(self, slot_index, filt_index,
                                   update_ana_filter=True):
@@ -786,6 +805,7 @@ def excepthook(etype, value, trace):
     exception = "".join([vinfo]+tmp)
 
     errorbox = QtWidgets.QMessageBox()
+    errorbox.setIcon(QtWidgets.QMessageBox.Critical)
     errorbox.addButton(QtWidgets.QPushButton('Close'),
                        QtWidgets.QMessageBox.YesRole)
     errorbox.addButton(QtWidgets.QPushButton(
